#ifndef HARKIO_HARK_HGTF_GTF_HPP_
#define HARKIO_HARK_HGTF_GTF_HPP_

#include <string>
#include <vector>
#include <complex>
#include <harkio2/Version.hpp>
#include <harkio2/Vector3.hpp>
#include <harkio2/defineitem.hpp>
#include <harkio2/hark/DataSet.hpp>
#include <harkio2/InvalidHarkFile.hpp>
#include <harkio2/hark/HarkFileWriter.hpp>

/** @file GTF.hpp
 * @brief GTF class and subclasses definitions.
 */

namespace harkio2
{
namespace hark
{
namespace hgtf
{

/** @brief Abstract GTF data set.
 * This class represents a set of transfer functions
 * from sound sources to microphones.
 */
class GTF : public harkio2::hark::DataSet
{
public:
    /** @brief Returns the type identifier.
     * @return The type identifier
     */
    virtual std::string GetId() const { return "hgtf::GTF"; }

    /** @brief Returns the version.
     * Returns the version which this instance actually represents.
     * @return The version number.
     */
    virtual harkio2::Version GetVersion() const = 0;
};

/** @brief GTF data set version 1.0.
 */
class GTF1_0 : public GTF
{
public:
    /** @brief Returns the version.
     * @return The version which this class represents.
     */
    virtual harkio2::Version GetVersion() const { return harkio2::Version(1, 0); }

    /** @brief Transfer function type.
     * This type represents a transfer function.
     */
    typedef std::vector<std::complex<float> > TFType;

    DEFINE_ITEM(nSource, unsigned int, NSource, unsigned int)
    DEFINE_ITEM(sourcePositions, std::vector<harkio2::Vector3<float> >,
            SourcePositions, const std::vector<harkio2::Vector3<float> >&)
    DEFINE_ITEM(nFIndex, unsigned int, NFIndex, unsigned int)
    DEFINE_ITEM(fIndexToBinIndexTable, std::vector<unsigned int>, FIndexToBinIndexTable, const std::vector<unsigned int>&)
    DEFINE_ITEM(tfs, std::vector<std::vector<TFType> >,
            TFs, const std::vector<std::vector<TFType> >&)

    /// @brief Implementation of Writable interface.
    void Write(HarkFileWriter& writer) throw (harkio2::InvalidHarkFile)
    {
        writer.Write(*this);
    }

};

}
}
}

#endif // HARKIO_HARK_HGTF_GTF_HPP_

